/**********************************************************************
* $Id$		Uart_Autobaud.c			2011-06-02
*//**
* @file		Uart_Autobaud.c
* @brief	This example describes how to configure UART using auto-baud
*			rate in interrupt mode
* @version	1.0
* @date		02. June. 2011
* @author	NXP MCU SW Application Team
* 
* Copyright(C) 2011, NXP Semiconductor
* All rights reserved.
*
***********************************************************************
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* products. This software is supplied "AS IS" without any warranties.
* NXP Semiconductors assumes no responsibility or liability for the
* use of the software, conveys no license or title under any patent,
* copyright, or mask work right to the product. NXP Semiconductors
* reserves the right to make changes in the software without
* notification. NXP Semiconductors also make no representation or
* warranty that such application will be suitable for the specified
* use without further testing or modification.
**********************************************************************/

#include "lpc177x_8x_uart.h"
#include "lpc177x_8x_pinsel.h"
#include "debug_frmwrk.h"
#include "EMC_NandFlash.h"
uint8_t ReadBuf[NANDFLASH_PAGE_FSIZE], WriteBuf[NANDFLASH_PAGE_FSIZE];
extern uint8_t InvalidBlockTable[NANDFLASH_NUMOF_BLOCK];

void GPIO_Configuration(void);
/*In <debug_frmwrk. H >file can choose to a serial port*/

int main(void)
{
    uint32_t FlashID;
    uint32_t i;
	/* Initialize debug via UART0
	 *  115200bps
	 *  8 data bit
	 *  No parity
	 *  1 stop bit
	 *  No flow control
	 */
	debug_frmwrk_init();
	_printf("NANDFLASH experiment\r\n");

    /* initialize memory */
    NandFlash_Init();
    _printf("Init NAND Flash...\r\n");
    NandFlash_Reset();
    _printf("RESET NAND Flash...\r\n");
    FlashID = NandFlash_ReadId();
//    if ( (FlashID & 0xFFFF0000) != K9FXX_ID )
//    {
//    	_DBG_("Error in reading NAND Flash ID, testing terminated!");
//    	while( 1 );	/* Fatal error */
//    }
    _printf("NAND Flash ID:0X%X!\r\n",FlashID);

    _printf("Checking valid block...\r\n");
    if ( NandFlash_ValidBlockCheck() == FALSE )
    {
    	_printf("Valid block checking error, testing terminated! At block(s): ");
    	///while ( 1 );		// Fatal error

		for(i = 0; i < NANDFLASH_NUMOF_BLOCK; i++)
		{
			if (InvalidBlockTable[i] == FALSE)
			{
				_printf("0X%X",i);
			}
		}
    }
    else
    _printf("Valid block checking OK\r\n");
    /**************************************************************
    * NandFlash_BlockErase() could scrub off all the invalid 	    *
    * block infomation including the factory initial invalid	    *
    * block table information. Per Samsung's K9F1G08 Users Manual,*
    * "Any intentional erasure of the initial invalid block		*
    * information is prohibited.									*
    *	However, during the driver debugging, it may create lot of  *
    *	invalid blocks. Below NandFlash_BlockErase() is used to deal *
    * with situation like that.                                   *
    *																*
    ***************************************************************/
    /* Erase the entire NAND FLASH */
    _printf("Erase entire NAND Flash...\r\n");
    for ( i = 0; i < NANDFLASH_NUMOF_BLOCK; i++ )
    {
	  	if ( NandFlash_BlockErase(i) == FALSE )
	  	{
			_printf("Erase NAND Flash fail at block: 0X%X",i);
	  	}
    }
    /* For the test program, the pattern for the whole page 2048 bytes
    is organized as: 0x0, 0x1, ... 0xFF, 0x0, 0x01...... */
    for ( i = 0; i < NANDFLASH_RW_PAGE_SIZE; i++ )
    {
	  	ReadBuf[i] = 0;
	  	WriteBuf[i] = i;
    }
    /* If it's a valid block, program all the pages of this block,
    read back, and finally validate. */
    /* simple program to program the block 0 and page 0 */

    if ( InvalidBlockTable[0] == 0 )
    {
    	_printf("Write a block of 2K data to NAND Flash...\r\n");
	  	if ( NandFlash_PageProgram( 0, 0, &WriteBuf[0] ) == FALSE )
		  	{
	  			_printf("Writing fail, testing terminated!\r\n");
	  			while ( 1 );	/* Fatal error */
		  	}

	  	_printf("Read back a block of 2K data from NAND Flash...\r\n");
	  	if ( NandFlash_PageRead( 0, 0, &ReadBuf[0] ) == FALSE )
		  	{
	  			_printf("Reading fail, testing terminated!\r\n");
	  			while ( 1 );	/* Fatal error */
		  	}

	  	/* Comparison read and write buffer */
	  	_printf("Verify data...\r\n");
	  	for ( i = 0; i < NANDFLASH_RW_PAGE_SIZE; i++ )
	  	{
			if ( ReadBuf[i] != WriteBuf[i] )
			{
			  	_printf("Verifying fail, testing terminated!\r\n");
				while ( 1 );	/* Fatal error */
			}
	  	}
    }

    _DBG_("Verifying complete! Testing terminated!");
    while (1);



	while(1);
}



#ifdef  DEBUG
/*******************************************************************************
* @brief		Reports the name of the source file and the source line number
* 				where the CHECK_PARAM error has occurred.
* @param[in]	file Pointer to the source file name
* @param[in]    line assert_param error line source number
* @return		None
*******************************************************************************/
void check_failed(uint8_t *file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while(1);
}
#endif

/*
 * @}
 */
