#include "EMC_NorFlash.h"
#include "lpc177x_8x_emc.h"
#include "lpc177x_8x_clkpwr.h"
#include "lpc177x_8x_pinsel.h"
#include "lpc177x_8x_timer.h"
#include "debug_frmwrk.h"


int32_t volatile timerdev = 0;

/*********************************************************************//**
 * @brief 		Delay
 * @param[in]	delayCnt Delay value
 * @return 		None
 **********************************************************************/
void delay(uint32_t delayCnt)
{
	uint32_t i;

	for ( i = 0; i < delayCnt; i++ );
	return;
}

/*********************************************************************//**
 * @brief 		Initialize external NOR FLASH memory
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void NORFLASHInit( void )
{
	TIM_TIMERCFG_Type TIM_ConfigStruct;
	/**************************************************************************
	* Initialize EMC for NOR FLASH
	**************************************************************************/
	EMC_Init();

    // init timer
	TIM_ConfigStruct.PrescaleOption = TIM_PRESCALE_USVAL;
	TIM_ConfigStruct.PrescaleValue	= 1;

		// Set configuration for Tim_config and Tim_MatchConfig
	TIM_Init(LPC_TIM0, TIM_TIMER_MODE,&TIM_ConfigStruct);
	TIM_Waitms(100);

	EMC_StaMemConfigMW (0,EMC_StaticConfig_MW_16BITS);
	EMC_StaMemConfigPB(0,EMC_StaticConfig_PB);
	EMC_SetStaMemoryParameter(0, EMC_STA_MEM_WAITWEN, EMC_StaticWaitWen_WAITWEN(2));
	EMC_SetStaMemoryParameter(0, EMC_STA_MEM_WAITOEN, EMC_StaticWaitOen_WAITOEN(2));
	EMC_SetStaMemoryParameter(0, EMC_STA_MEM_WAITRD, EMC_StaticWaitRd_WAITRD(0x1f));
	EMC_SetStaMemoryParameter(0, EMC_STA_MEM_WAITPAGE, EMC_StaticwaitPage_WAITPAGE(0x1f));
	EMC_SetStaMemoryParameter(0, EMC_STA_MEM_WAITWR, EMC_StaticWaitwr_WAITWR(0x1f));
	EMC_SetStaMemoryParameter(0, EMC_STA_MEM_WAITTURN, EMC_StaticWaitTurn_WAITTURN(0x1f));

	//delay time
 	TIM_Waitms(10);

  	return;
}

/*********************************************************************//**
 * @brief 		Toggle Bit check if the data is written or erased
 * @param[in]	Addr	address value
 * @param[in]	Data	expected data
 * @return 		Checking result, could be:
 * 					- TRUE: Done
 *					- FALSE: Timeout
 **********************************************************************/
uint32_t ToggleBitCheck( uint32_t Addr, uint16_t Data )
{
	volatile uint16_t *ip;
	uint16_t temp1, temp2;
	uint32_t TimeOut = PROGRAM_TIMEOUT;

	while( TimeOut > 0 )
	{
		ip = GET_ADDR(Addr);
		temp1 = *ip;
		ip = GET_ADDR(Addr);
		temp2 = *ip;

		if ( (temp1 == temp2) && (temp1 == Data) )
		{
		 	return( TRUE );
		}
		TimeOut--;
	}
	return ( FALSE );
}

/*********************************************************************//**
 * @brief 		Check ID from external NOR FLASH memory
 * @param[in]	None
 * @return 		Checking result, could be:
 * 					- TRUE: Correct
 *					- FALSE: Incorrect
 **********************************************************************/
uint32_t NORFLASHCheckID( void )
{
  volatile uint16_t *ip;
  uint16_t SST_id1=0, SST_id2=0;

  /*  Issue the Software Product ID code to 39VF160   */
  ip  = GET_ADDR(0x5555);
  *ip = 0x00AA;
  ip  = GET_ADDR(0x2AAA);
  *ip = 0x0055;
  ip  = GET_ADDR(0x5555);
  *ip = 0x0090;
  delay(10);

  /* Read the product ID from 39VF160 */
  ip  = GET_ADDR(0x0000);
  SST_id1 = *ip & 0x00FF;
  ip  = GET_ADDR(0x0001);
  SST_id2 = *ip;

  /* Issue the Soffware Product ID Exit code thus returning the 39VF160 */
  /* to the read operating mode */
  ip  = GET_ADDR(0x5555);
  *ip = 0x00AA;
  ip  = GET_ADDR(0x2AAA);
  *ip = 0x0055;
  ip  = GET_ADDR(0x5555);
  *ip = 0x00F0;
  delay(10); 

  _printf("\r\nSST_id1=0X%X",SST_id1);
  _printf("\r\nSST_id2=0X%X",SST_id2);
  _printf("\r\n");

  /* Check ID */
  if ((SST_id1 == SST_ID) && (SST_id2 == SST_39VF320))
	return( TRUE );
  else
	return( FALSE );

}


/*********************************************************************//**
 * @brief 		Erase external NOR FLASH memory
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void NORFLASHErase( void )
{
  volatile uint16_t *ip;

  ip  = GET_ADDR(0x5555);
  *ip = 0x00AA;
  ip  = GET_ADDR(0x2AAA);
  *ip = 0x0055;
  ip  = GET_ADDR(0x5555);
  *ip = 0x0080;
  ip  = GET_ADDR(0x5555);
  *ip = 0x00AA;
  ip  = GET_ADDR(0x2AAA);
  *ip = 0x0055;
  ip  = GET_ADDR(0x5555);
  *ip = 0x0010;
  delay(10000000);				/* Use timer 1 */
  return;

}

/*********************************************************************//**
 * @brief 		Program one 16-bit data into external NOR FLASH memory
 *				This "uint16_t" for the external flash is 16 bits!!!
 * @param[in]	Addr	Address value
 * @param[in]	Data	data value
 * @return 		Program result, could be:
 					- TRUE: succesful
					- FALSE: fail
 **********************************************************************/
uint32_t NORFLASHWriteWord( uint32_t Addr, uint16_t Data )
{
  volatile uint16_t *ip;

  ip  = GET_ADDR(0x5555);
  *ip = 0x00AA;
  ip  = GET_ADDR(0x2aaa);
  *ip = 0x0055;
  ip  = GET_ADDR(0x5555);
  *ip = 0x00A0;

  ip = GET_ADDR(Addr);		/* Program 16-bit word */
  *ip = Data;
  return ( ToggleBitCheck( Addr, Data ) );
}

/*********************************************************************************
**                            End Of File
*********************************************************************************/
