/**********************************************************************
* $Id$		Uart_Autobaud.c			2011-06-02
*//**
* @file		Uart_Autobaud.c
* @brief	This example describes how to configure UART using auto-baud
*			rate in interrupt mode
* @version	1.0
* @date		02. June. 2011
* @author	NXP MCU SW Application Team
* 
* Copyright(C) 2011, NXP Semiconductor
* All rights reserved.
*
***********************************************************************
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* products. This software is supplied "AS IS" without any warranties.
* NXP Semiconductors assumes no responsibility or liability for the
* use of the software, conveys no license or title under any patent,
* copyright, or mask work right to the product. NXP Semiconductors
* reserves the right to make changes in the software without
* notification. NXP Semiconductors also make no representation or
* warranty that such application will be suitable for the specified
* use without further testing or modification.
**********************************************************************/

#include "lpc177x_8x_uart.h"
#include "lpc177x_8x_pinsel.h"
#include "debug_frmwrk.h"
#include "EMC_NorFlash.h"
#include "system_LPC177x_8x.h"

#define NORFLASH_RW_PAGE_SIZE	0x800

void GPIO_Configuration(void);
/*In <debug_frmwrk. H >file can choose to a serial port*/

int main(void)
{

    uint32_t i,ID;
    volatile uint16_t *ip;
    uint32_t NorFlashAdr;
    SystemInit();

	debug_frmwrk_init();
    _printf("Init NorFlash...\r\n");
    NORFLASHInit();

    _printf("Read NOR Flash ID...\r\n");
    if ( NORFLASHCheckID() == FALSE )
    {
    	_printf("Error in reading NOR Flash ID, testing terminated!\r\n");
    	while ( 1 );		/* Fatal error */
    }
   _printf("Erase entire NOR Flash...\r\n");
    NORFLASHErase();		/* Chip erase */

    /* Write to flash with pattern 0xAA55 and 0xA55A */
    NorFlashAdr = NOR_FLASH_BASE;

    _printf("Write a block of 2K data to NOR Flash...\r\n");
    for ( i = 0; i < NORFLASH_RW_PAGE_SIZE/2; i++ )
    {
	  	NORFLASHWriteWord( NorFlashAdr, 0xAA55 );
	  	NorFlashAdr++;
	  	NORFLASHWriteWord( NorFlashAdr, 0xA55A );
	  	NorFlashAdr++;
    }

    /* Verify */
    _printf("Verify data...\r\n");
    NorFlashAdr = NOR_FLASH_BASE;
    for ( i = 0; i < NORFLASH_RW_PAGE_SIZE/2; i+=2 )
    {
	  	ip  = GET_ADDR(i);
	  	if ( (*ip & 0xFFFF) != 0xAA55 )
	  	{
	  		_printf("Verifying fail, testing terminated!\r\n");
	  		while ( 1 );	/* Fatal error */
	  	}

	  	ip  = GET_ADDR(i+1);
	  	if ( (*ip & 0xFFFF) != 0xA55A )
	  	{
	  		_printf("Verifying fail, testing terminated!\r\n");
	  		while ( 1 );	/* Fatal error */
	  	}
    }

    // terminated
    _printf("Verifying complete! Testing terminated!\r\n");

    while (1);
}





#ifdef  DEBUG
/*******************************************************************************
* @brief		Reports the name of the source file and the source line number
* 				where the CHECK_PARAM error has occurred.
* @param[in]	file Pointer to the source file name
* @param[in]    line assert_param error line source number
* @return		None
*******************************************************************************/
void check_failed(uint8_t *file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while(1);
}
#endif

/*
 * @}
 */
